@@ -44,6 +44,10 @@
 #endif
 #endif
 
+#if SIGNAL_OKAY
+#include <signal.h>
+#endif
+
 #define DMALLOC_DISABLE
 
 #include "dmalloc.h"
@@ -60,50 +64,51 @@
 #include "dmalloc_lp.h"
 
 #if INCLUDE_RCS_IDS
-LOCAL	char	*rcs_id =
-  "$Id: malloc.c,v 1.88 1995/10/21 00:23:51 gray Exp $";
+static	char	*rcs_id =
+  "$Id: malloc.c,v 1.101 1997/12/22 00:25:47 gray Exp $";
 #endif
 
 /*
  * exported variables
  */
-/* logfile for dumping dmalloc info, DMALLOC_LOGFILE env var overrides this */
-EXPORT	char		*dmalloc_logpath = LOGPATH_INIT;
-/* internal dmalloc error number for reference purposes only */
-EXPORT	int		dmalloc_errno = ERROR_NONE;
-/* address to look for.  when discovered call dmalloc_error() */
-EXPORT	DMALLOC_PNT	dmalloc_address = ADDRESS_INIT;
 /*
  * argument to dmalloc_address, if 0 then never call dmalloc_error()
  * else call it after seeing dmalloc_address for this many times.
  */
-EXPORT	int		dmalloc_address_count = ADDRESS_COUNT_INIT;
+int		dmalloc_address_count = ADDRESS_COUNT_INIT;
 
 /* local routines */
-LOCAL	int		dmalloc_startup(void);
-EXPORT	void		_dmalloc_shutdown(void);
-EXPORT	void		_dmalloc_log_heap_map(void);
-EXPORT	void		_dmalloc_log_stats(void);
-EXPORT	void		_dmalloc_log_unfreed(void);
-EXPORT	int		_dmalloc_verify(const DMALLOC_PNT pnt);
-EXPORT	int		_malloc_verify(const DMALLOC_PNT pnt);
-EXPORT	void		_dmalloc_debug(const int debug);
-EXPORT	int		_dmalloc_debug_current(void);
-EXPORT	int		_dmalloc_examine(const DMALLOC_PNT pnt, DMALLOC_SIZE * size,
-					 char ** file, unsigned int * line,
-					 DMALLOC_PNT * ret_attr);
-EXPORT	char		*_dmalloc_strerror(const int errnum);
+static	int		dmalloc_startup(void);
+void		_dmalloc_shutdown(void);
+void		_dmalloc_log_heap_map(void);
+void		_dmalloc_log_stats(void);
+void		_dmalloc_log_unfreed(void);
+int		_dmalloc_verify(const DMALLOC_PNT pnt);
+int		_malloc_verify(const DMALLOC_PNT pnt);
+void		_dmalloc_debug(const int flags);
+int		_dmalloc_debug_current(void);
+int		_dmalloc_examine(const DMALLOC_PNT pnt, DMALLOC_SIZE * size,
+				 char ** file, unsigned int * line,
+				 DMALLOC_PNT * ret_attr);
+const char	*_dmalloc_strerror(const int error_num);
 
 
 /* local variables */
-LOCAL	char		enabled		= FALSE; /* have we started yet? */
-LOCAL	char		in_alloc	= FALSE; /* can't be here twice */
+static	int		enabled_b	= FALSE; /* have we started yet? */
+static	int		in_alloc_b	= FALSE; /* can't be here twice */
+static	int		do_shutdown_b	= FALSE; /* execute shutdown soon */
+
+#ifdef THREAD_LOCK_GLOBAL
+/* define the global thread-lock variable(s) if needed */
+THREAD_LOCK_GLOBAL
+#endif
 
 /* debug variables */
-LOCAL	char		*start_file = START_FILE_INIT; /* file to start at */
-LOCAL	int		start_line = START_LINE_INIT; /* line to start */
-LOCAL	int		start_count = START_COUNT_INIT; /* start after X */
-LOCAL	int		check_interval = INTERVAL_INIT; /* check every X */
+static	char		*start_file = START_FILE_INIT; /* file to start at */
+static	int		start_line = START_LINE_INIT; /* line to start */
+static	int		start_count = START_COUNT_INIT; /* start after X */
+static	int		check_interval = INTERVAL_INIT; /* check every X */
+static	int		thread_lock_on = LOCK_ON_INIT;	/* turn locking on */
 
 /****************************** local utilities ******************************/
 
@@ -111,12 +116,26 @@ LOCAL	int		check_interval = INTERVAL_INIT; /* check every X */
  * a call to the alloc routines has been made, check the debug variables
  * returns ERROR or NOERROR.
  */
-LOCAL	int	check_debug_vars(const char * file, const int line)
+static	int	check_debug_vars(const char * file, const int line)
 {
-  if (_dmalloc_aborting)
+  if (_dmalloc_aborting_b) {
     return ERROR;
+  }
   
-  if (in_alloc) {
+  /*
+   * NOTE: we need to do this outside of lock to get env vars
+   * otherwise our thread_lock_on variable won't be initialized and
+   * the THREAD_LOCK will flip.
+   */
+  if (! enabled_b) {
+    if (dmalloc_startup() != NOERROR) {
+      return ERROR;
+    }
+  }
+  
+  THREAD_LOCK;
+  
+  if (in_alloc_b) {
     dmalloc_errno = ERROR_IN_TWICE;
     dmalloc_error("check_debug_vars");
     /* NOTE: dmalloc_error may die already */
@@ -124,13 +143,7 @@ LOCAL	int	check_debug_vars(const char * file, const int line)
     /*NOTREACHED*/
   }
   
-  THREAD_LOCK;
-  
-  in_alloc = TRUE;
-  
-  if (! enabled)
-    if (dmalloc_startup() != NOERROR)
-      return ERROR;
+  in_alloc_b = TRUE;
   
   /* check start file/line specifications */
   if (! BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)
@@ -138,25 +151,30 @@ LOCAL	int	check_debug_vars(const char * file, const int line)
       && file != DMALLOC_DEFAULT_FILE
       && line != DMALLOC_DEFAULT_LINE
       && strcmp(start_file, file) == 0
-      && (start_line == 0 || start_line == line))
+      && (start_line == 0 || start_line == line)) {
     BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
+  }
   
   /* start checking heap after X times */
-  if (start_count != START_COUNT_INIT && --start_count == 0)
+  if (start_count != START_COUNT_INIT && --start_count == 0) {
     BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
+  }
   
   /* checking heap every X times */
-  _dmalloc_iterc++;
+  _dmalloc_iter_c++;
   if (check_interval != INTERVAL_INIT && check_interval > 0) {
-    if (_dmalloc_iterc % check_interval == 0)
+    if (_dmalloc_iter_c % check_interval == 0) {
       BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
-    else
+    }
+    else { 
       BIT_CLEAR(_dmalloc_flags, DEBUG_CHECK_HEAP);
+    }
   }
   
   /* after all that, do we need to check the heap? */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP))
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)) {
     (void)_chunk_check();
+  }
   
   return NOERROR;
 }
@@ -165,13 +183,14 @@ LOCAL	int	check_debug_vars(const char * file, const int line)
  * check out a pointer to see if we were looking for it.  this should
  * be re-entrant and it may not return.
  */
-LOCAL	void	check_pnt(const char * file, const int line, const void * pnt,
+static	void	check_pnt(const char * file, const int line, const void * pnt,
 			  const char * label)
 {
   static int	addc = 0;
   
-  if (dmalloc_address == ADDRESS_INIT || pnt != dmalloc_address)
+  if (dmalloc_address == ADDRESS_INIT || pnt != dmalloc_address) {
     return;
+  }
   
   addc++;
   _dmalloc_message("address '%#lx' found in '%s' at pass %d from '%s'",
@@ -186,41 +205,62 @@ LOCAL	void	check_pnt(const char * file, const int line, const void * pnt,
 }
 
 /*
- * process the values of dmalloc environ variable(s)
+ * process the values of dmalloc environ variables
  */
-LOCAL	void	process_environ(void)
+static	void	process_environ(void)
 {
   _dmalloc_environ_get(OPTIONS_ENVIRON, (unsigned long *)&dmalloc_address,
 		       &dmalloc_address_count, &_dmalloc_flags,
-		       &check_interval, &dmalloc_logpath,
+		       &check_interval, &thread_lock_on, &dmalloc_logpath,
 		       &start_file, &start_line, &start_count);
   
   /* if it was not set then no flags set */
-  if (_dmalloc_flags == DEBUG_INIT)
+  if (_dmalloc_flags == DEBUG_INIT) {
     _dmalloc_flags = 0;
+  }
   
   /* if we set the start stuff, then check-heap comes on later */
-  if (start_count != -1)
+  if (start_count != -1) {
     BIT_CLEAR(_dmalloc_flags, DEBUG_CHECK_HEAP);
+  }
   
   /* override dmalloc_debug value if we we call dmalloc_debug() already */
-  if (_dmalloc_debug_preset != DEBUG_PRE_NONE)
+  if (_dmalloc_debug_preset != DEBUG_PRE_NONE) {
     _dmalloc_flags = _dmalloc_debug_preset;
+  }
 }
 
 /************************** startup/shutdown calls ***************************/
 
+#if SIGNAL_OKAY
+/*
+ * signal catcher
+ */
+static	RETSIGTYPE	signal_handler(const int sig)
+{
+  _dmalloc_message("caught signal %d", sig);
+  /* if we are already inside malloc then do the shutdown later */
+  if (in_alloc_b) {
+    do_shutdown_b = TRUE;
+  }
+  else {
+    _dmalloc_shutdown();
+  }
+}
+#endif
+
 /*
  * startup the memory-allocation module
  */
-LOCAL	int	dmalloc_startup(void)
+static	int	dmalloc_startup(void)
 {
   /* have we started already? */
-  if (enabled)
+  if (enabled_b) {
     return ERROR;
+  }
   
   /* set this here so if an error occurs below, it will not try again */
-  enabled = TRUE;
+  enabled_b = TRUE;
   
 #if STORE_TIMEVAL
   GET_TIMEVAL(_dmalloc_start);
@@ -234,12 +274,14 @@ LOCAL	int	dmalloc_startup(void)
   process_environ();
   
   /* startup heap code */
-  if (_heap_startup() == ERROR)
+  if (_heap_startup() == ERROR) {
     return ERROR;
+  }
   
   /* startup the chunk lower-level code */
-  if (_chunk_startup() == ERROR)
+  if (_chunk_startup() == ERROR) {
     return ERROR;
+  }
   
   /* set leap variables */
   _dmalloc_shutdown_func = _dmalloc_shutdown;
@@ -253,18 +295,17 @@ LOCAL	int	dmalloc_startup(void)
   _dmalloc_examine_func = _dmalloc_examine;
   _dmalloc_strerror_func = _dmalloc_strerror;
   
+  /*
+   * NOTE: we may go recursive below here becasue atexit or on_exit
+   * may ask for memory to be allocated.  We won't worry about it and
+   * will just give it to them.  We hope that atexit didn't start the
+   * allocating.  Ugh.
+   */
 #if AUTO_SHUTDOWN
   {
     unsigned int	line_hold = _dmalloc_line;
     char		*file_hold = _dmalloc_file;
     
-    /*
-     * HACK: we have to disable the in_alloc because we might be about
-     * to go recursize.  this should not get back here since dmalloc
-     * has been enabled.
-     */
-    in_alloc = FALSE;
-    
     /* NOTE: I use the else here in case some dumb systems has both */
 #if HAVE_ATEXIT
     (void)atexit(_dmalloc_shutdown);
@@ -274,13 +315,34 @@ LOCAL	int	dmalloc_startup(void)
 #endif
 #endif
     
-    in_alloc = TRUE;
-    
     _dmalloc_line = line_hold;
     _dmalloc_file = file_hold;
   }
 #endif /* AUTO_SHUTDOWN */
   
+#if SIGNAL_OKAY
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CATCH_SIGNALS)) {
+#ifdef SIGNAL1
+    (void)signal(SIGNAL1, signal_handler);
+#endif
+#ifdef SIGNAL2
+    (void)signal(SIGNAL2, signal_handler);
+#endif
+#ifdef SIGNAL3
+    (void)signal(SIGNAL3, signal_handler);
+#endif
+#ifdef SIGNAL4
+    (void)signal(SIGNAL4, signal_handler);
+#endif
+#ifdef SIGNAL5
+    (void)signal(SIGNAL5, signal_handler);
+#endif
+#ifdef SIGNAL6
+    (void)signal(SIGNAL6, signal_handler);
+#endif
+  }
+#endif /* SIGNAL_OKAY */
+  
   return NOERROR;
 }
 
@@ -288,38 +350,48 @@ LOCAL	int	dmalloc_startup(void)
  * shutdown memory-allocation module, provide statistics if necessary
  * NOTE: called by way of leap routine in dmalloc_lp.c
  */
-EXPORT	void	_dmalloc_shutdown(void)
+void	_dmalloc_shutdown(void)
 {
   /* NOTE: do not generate errors for IN_TWICE here */
   
-  /* if we've died in dmalloc somewhere then leave fast and quietly */
-  if (in_alloc || _dmalloc_aborting)
+  /* if we're already in die mode leave fast and quietly */
+  if (_dmalloc_aborting_b) {
     return;
-  
-  in_alloc = TRUE;
+  }
   
   THREAD_LOCK;
   
+  /* if we've died in dmalloc somewhere then leave fast and quietly */
+  if (in_alloc_b) {
+    return;
+  }
+  
+  in_alloc_b = TRUE;
+  
   /* check the heap since we are dumping info from it */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP))
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)) {
     (void)_chunk_check();
+  }
   
   /* dump some statistics to the logfile */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_STATS))
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_STATS)) {
     _chunk_list_count();
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_STATS))
+  }
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_STATS)) {
     _chunk_stats();
+  }
   
   /* report on non-freed pointers */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_NONFREE))
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_NONFREE)) {
     _chunk_dump_unfreed();
+  }
   
 #if STORE_TIMEVAL
   {
     struct timeval	now;
     GET_TIMEVAL(now);
     _dmalloc_message("ending time = %ld.%ld, elapsed since start = %s",
-		     now.tv_sec, now.tv_usec, _dmalloc_ptime(&now, TRUE));
+		     now.tv_sec, now.tv_usec, _dmalloc_ptimeval(&now, TRUE));
   }
 #else
 #if HAVE_TIME
@@ -331,46 +403,65 @@ EXPORT	void	_dmalloc_shutdown(void)
 #endif
 #endif
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   THREAD_UNLOCK;
   
-  /* NOTE: do not set enabled to false here */
+  /* NOTE: do not set enabled_b to false here */
 }
 
+#if FINI_DMALLOC
+/*
+ * Automatic OSF function to close dmalloc.  Pretty cool OS/compiler
+ * hack.  By default it is not necessary because we use atexit() and
+ * on_exit() to register the close functions.  These are more
+ * portable.
+ */
+void	__fini_dmalloc()
+{
+  dmalloc_shutdown();
+}
+#endif
+
 /******************************* memory calls ********************************/
 
 /*
  * allocate and return a SIZE block of bytes.  returns 0L on error.
  */
 #undef malloc
-EXPORT	DMALLOC_PNT	malloc(DMALLOC_SIZE size)
+DMALLOC_PNT	malloc(DMALLOC_SIZE size)
 {
-  void		*newp;
+  void		*new_p;
   
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
-  /* yes, this could be always false if size is unsigned */
+#if DMALLOC_SIZE_UNSIGNED == 0
   if (size < 0) {
     dmalloc_errno = ERROR_BAD_SIZE;
     dmalloc_error("malloc");
     return MALLOC_ERROR;
   }
+#endif
   
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
     return MALLOC_ERROR;
+  }
   
-  newp = _chunk_malloc(_dmalloc_file, _dmalloc_line, size);
-  in_alloc = FALSE;
+  new_p = _chunk_malloc(_dmalloc_file, _dmalloc_line, size);
+  in_alloc_b = FALSE;
   
-  check_pnt(_dmalloc_file, _dmalloc_line, newp, "malloc");
+  check_pnt(_dmalloc_file, _dmalloc_line, new_p, "malloc");
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
   
-  return newp;
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+  
+  return new_p;
 }
 
 /*
@@ -379,9 +470,9 @@ EXPORT	DMALLOC_PNT	malloc(DMALLOC_SIZE size)
  * error.
  */
 #undef calloc
-EXPORT	DMALLOC_PNT	calloc(DMALLOC_SIZE num_elements, DMALLOC_SIZE size)
+DMALLOC_PNT	calloc(DMALLOC_SIZE num_elements, DMALLOC_SIZE size)
 {
-  void		*newp;
+  void		*new_p;
   DMALLOC_SIZE	len = num_elements * size;
   
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
@@ -389,54 +480,79 @@ EXPORT	DMALLOC_PNT	calloc(DMALLOC_SIZE num_elements, DMALLOC_SIZE size)
   /* needs to be done here */
   _calloc_count++;
   
-  newp = malloc(len);
-  if (newp != MALLOC_ERROR && len > 0)
-    (void)memset(newp, NULLC, len);
+  new_p = malloc(len);
+  if (new_p != MALLOC_ERROR && len > 0) {
+    (void)memset(new_p, 0, len);
+  }
   
-  return newp;
+  return new_p;
 }
 
 /*
- * resizes OLD_PNT to NEW_SIZE bytes and return the new space after
- * either copying all of OLD_PNT to the new area or truncating.
- * returns 0L on error.
+ * Resizes OLD_PNT to NEW_SIZE bytes and return the new space after
+ * either copying all of OLD_PNT to the new area or truncating.  If
+ * OLD_PNT is 0L then it will do the equivalent of malloc(NEW_SIZE).
+ * If NEW_SIZE is 0 and OLD_PNT is not 0L then it will do the
+ * equivalent of free(OLD_PNT) and will return 0L.  Returns 0L on
+ * error.
  */
 #undef realloc
-EXPORT	DMALLOC_PNT	realloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
+DMALLOC_PNT	realloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
 {
-  void		*newp;
+  void		*new_p;
   
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
-  /* yes, this could be always false if new_size is unsigned */
+#if DMALLOC_SIZE_UNSIGNED == 0
   if (new_size < 0) {
     dmalloc_errno = ERROR_BAD_SIZE;
     dmalloc_error("realloc");
     return MALLOC_ERROR;
   }
+#endif
+  
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
+    return REALLOC_ERROR;
+  }
   
   check_pnt(_dmalloc_file, _dmalloc_line, old_pnt, "realloc-in");
   
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
-    return REALLOC_ERROR;
-  
 #if ALLOW_REALLOC_NULL
-  if (old_pnt == NULL)
-    newp = _chunk_malloc(_dmalloc_file, _dmalloc_line, new_size);
+  if (old_pnt == NULL) {
+    new_p = _chunk_malloc(_dmalloc_file, _dmalloc_line, new_size);
+  }
   else
 #endif
-    newp = _chunk_realloc(_dmalloc_file, _dmalloc_line, old_pnt, new_size);
+#if ALLOW_REALLOC_SIZE_ZERO
+    if (new_size == 0) {
+      /*
+       * If realloc(old_pnt, 0) then free(old_pnt).  Thanks to
+       * Stefan.Froehlich@tuwien.ac.at for patiently pointing that the
+       * realloc in just about every Unix has this functionality.
+       */
+      (void)_chunk_free(_dmalloc_file, _dmalloc_line, old_pnt);
+      new_p = NULL;
+    }
+    else
+#endif
+      new_p = _chunk_realloc(_dmalloc_file, _dmalloc_line, old_pnt, new_size);
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
-  check_pnt(_dmalloc_file, _dmalloc_line, newp, "realloc-out");
+  if (new_p != NULL) {
+    check_pnt(_dmalloc_file, _dmalloc_line, new_p, "realloc-out");
+  }
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
   
-  return newp;
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+  
+  return new_p;
 }
 
 /*
@@ -444,14 +560,12 @@ EXPORT	DMALLOC_PNT	realloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
  * depending on whether STDC is defined by your compiler.
  */
 #undef free
-EXPORT	DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
+DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
 {
   int		ret;
   
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
-  check_pnt(_dmalloc_file, _dmalloc_line, pnt, "free");
-  
   if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
 #if defined(__STDC__) && __STDC__ == 1
     return;
@@ -460,6 +574,8 @@ EXPORT	DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
 #endif
   }
   
+  check_pnt(_dmalloc_file, _dmalloc_line, pnt, "free");
+  
 #if ALLOW_FREE_NULL
   if (pnt == NULL) {
 #if ALLOW_FREE_NULL_MESSAGE
@@ -474,13 +590,17 @@ EXPORT	DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
   ret = _chunk_free(_dmalloc_file, _dmalloc_line, pnt);
 #endif
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
   
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+  
 #if defined(__STDC__) && __STDC__ == 1
 #else
   return ret;
@@ -488,10 +608,10 @@ EXPORT	DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
 }
 
 /*
- * same as free(PNT)
+ * same as free PNT
  */
 #undef cfree
-EXPORT	DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
+DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
 {
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
@@ -508,68 +628,84 @@ EXPORT	DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
  * log the heap structure plus information on the blocks if necessary.
  * NOTE: called by way of leap routine in dmalloc_lp.c
  */
-EXPORT	void	_dmalloc_log_heap_map(void)
+void	_dmalloc_log_heap_map(void)
 {
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
   /* check the heap since we are dumping info from it */
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
     return;
+  }
   
   _chunk_log_heap_map();
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
+  
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
 }
 
 /*
  * dump dmalloc statistics to logfile
  * NOTE: called by way of leap routine in dmalloc_lp.c
  */
-EXPORT	void	_dmalloc_log_stats(void)
+void	_dmalloc_log_stats(void)
 {
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
     return;
+  }
   
   _chunk_list_count();
   _chunk_stats();
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
+  
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
 }
 
 /*
  * dump unfreed-memory info to logfile
  * NOTE: called by way of leap routine in dmalloc_lp.c
  */
-EXPORT	void	_dmalloc_log_unfreed(void)
+void	_dmalloc_log_unfreed(void)
 {
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
     return;
+  }
   
-  if (! BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS))
+  if (! BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS)) {
     _dmalloc_message("dumping the unfreed pointers");
+  }
   
   _chunk_dump_unfreed();
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
+  
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
 }
 
 /*
@@ -577,7 +713,7 @@ EXPORT	void	_dmalloc_log_unfreed(void)
  * NOTE: called by way of leap routine in dmalloc_lp.c
  * returns MALLOC_VERIFY_ERROR or MALLOC_VERIFY_NOERROR
  */
-EXPORT	int	_dmalloc_verify(const DMALLOC_PNT pnt)
+int	_dmalloc_verify(const DMALLOC_PNT pnt)
 {
   int	ret;
   
@@ -585,65 +721,75 @@ EXPORT	int	_dmalloc_verify(const DMALLOC_PNT pnt)
   
   /* should not check heap here because we will be doing it below */
   
-  if (_dmalloc_aborting)
+  if (_dmalloc_aborting_b) {
     return DMALLOC_VERIFY_ERROR;
+  }
   
-  if (in_alloc) {
+  THREAD_LOCK;
+  
+  if (in_alloc_b) {
     dmalloc_errno = ERROR_IN_TWICE;
-    dmalloc_error("check_debug_vars");
+    dmalloc_error("dmalloc_verify");
     /* NOTE: dmalloc_error may die already */
     _dmalloc_die(FALSE);
     /*NOTREACHED*/
   }
   
-  in_alloc = TRUE;
+  in_alloc_b = TRUE;
   
-  THREAD_UNLOCK;
-  
-  if (pnt == NULL)
+  if (pnt == NULL) {
     ret = _chunk_check();
-  else
+  }
+  else {
     ret = _chunk_pnt_check("dmalloc_verify", pnt, CHUNK_PNT_EXACT, 0);
+  }
   
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
   
-  if (ret == NOERROR)
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+  
+  if (ret == NOERROR) {
     return DMALLOC_VERIFY_NOERROR;
-  else
+  }
+  else {
     return DMALLOC_VERIFY_ERROR;
+  }
 }
 
 /*
  * same as dmalloc_verify
  */
-EXPORT	int	_malloc_verify(const DMALLOC_PNT pnt)
+int	_malloc_verify(const DMALLOC_PNT pnt)
 {
   return _dmalloc_verify(pnt);
 }
 
 /*
- * set the global debug functionality flags to DEBUG (0 to disable all
+ * set the global debug functionality FLAGS (0 to disable all
  * debugging).  NOTE: after this module has started up, you cannot set
  * certain flags such as fence-post or free-space checking.
  */
-EXPORT	void	_dmalloc_debug(const int debug)
+void	_dmalloc_debug(const int flags)
 {
   /* should not check the heap here since we are setting the debug variable */
   
   /* if we've not started up then set the variable */
-  if (! enabled)
-    _dmalloc_flags = debug;
+  if (! enabled_b) {
+    _dmalloc_flags = flags;
+  }
   else {
     /* make sure that the not-changeable flag values are preserved */
     _dmalloc_flags &= DEBUG_NOT_CHANGEABLE;
     
     /* add the new flags - the not-addable ones */
-    _dmalloc_flags |= debug & ~DEBUG_NOT_ADDABLE;
+    _dmalloc_flags |= flags & ~DEBUG_NOT_ADDABLE;
   }
 }
 
@@ -651,7 +797,7 @@ EXPORT	void	_dmalloc_debug(const int debug)
  * returns the current debug functionality flags.  this allows you to
  * save a dmalloc library state to be restored later.
  */
-EXPORT	int	_dmalloc_debug_current(void)
+int	_dmalloc_debug_current(void)
 {
   /* should not check the heap here since we are dumping the debug variable */
   return _dmalloc_flags;
@@ -663,9 +809,9 @@ EXPORT	int	_dmalloc_debug_current(void)
  * if FILE returns 0L then RET_ATTR may have a value and vice versa.
  * returns NOERROR or ERROR depending on whether PNT is good or not
  */
-EXPORT	int	_dmalloc_examine(const DMALLOC_PNT pnt, DMALLOC_SIZE * size,
-				 char ** file, unsigned int * line,
-				 DMALLOC_PNT * ret_attr)
+int	_dmalloc_examine(const DMALLOC_PNT pnt, DMALLOC_SIZE *size,
+			 char **file, unsigned int *line,
+			 DMALLOC_PNT *ret_attr)
 {
   int		ret;
   unsigned int	size_map;
@@ -673,38 +819,51 @@ EXPORT	int	_dmalloc_examine(const DMALLOC_PNT pnt, DMALLOC_SIZE * size,
   SET_RET_ADDR(_dmalloc_file, _dmalloc_line);
   
   /* need to check the heap here since we are geting info from it below */
-  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR)
+  if (check_debug_vars(_dmalloc_file, _dmalloc_line) != NOERROR) {
     return ERROR;
+  }
   
   /* NOTE: we do not need the alloc-size info */
   ret = _chunk_read_info(pnt, &size_map, NULL, file, line, ret_attr,
 			 "dmalloc_examine", NULL);
-  in_alloc = FALSE;
+  in_alloc_b = FALSE;
   
   _dmalloc_file = DMALLOC_DEFAULT_FILE;
   _dmalloc_line = DMALLOC_DEFAULT_LINE;
   
   THREAD_UNLOCK;
   
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+  
   if (ret == NOERROR) {
-    if (size != NULL)
+    if (size != NULL) {
       *size = size_map;
+    }
     return NOERROR;
   }
-  else
+  else {
     return ERROR;
+  }
 }
 
 /*
- * dmalloc version of strerror to return the string version of ERRNUM
- * returns the string for MALLOC_BAD_ERRNO if ERRNUM is out-of-range.
+ * Dmalloc version of strerror to return the string version of
+ * ERROR_NUM.  Returns an invaid errno string if ERROR_NUM is
+ * out-of-range.
  */
-EXPORT	char	*_dmalloc_strerror(const int errnum)
+const char	*_dmalloc_strerror(const int error_num)
 {
+  error_str_t	*err_p;
+  
   /* should not check_debug_vars here because _dmalloc_error calls this */
   
-  if (! IS_MALLOC_ERRNO(errnum))
-    return errlist[ERROR_BAD_ERRNO];
-  else
-    return errlist[errnum];
+  for (err_p = error_list; err_p->es_error != 0; err_p++) {
+    if (err_p->es_error == error_num) {
+      return err_p->es_string;
+    }
+  }
+  
+  return INVALID_ERROR;
 }
